# subscribe_api.py
import fire
from pybraries.subscription_helpers import sub_api


class Subscribe(object):
    """Class for libraries.io API for changing user's libraries.io subscriptions"""

    def __init__(self):
        pass

    def list_subscribed(self, *args, **kwargs):
        """
        Return a list of packages a user is subscribed to for release notifications.

        Args:

        Returns:
            response (dict): dict response from libraries.io
        """
        return sub_api("list_subscribed", *args, **kwargs)

    def subscribe(self, *args, **kwargs):
        """
        Subscribe to receive notifications about new releases of a project.

        Args:
            manager (str): package manager name (e.g. PyPI)
            package (str): package name
            include_prerelease (bool): default = True. Include prerelease notifications
        Returns:
            response (dict): dict of project info from libraries.io
        """
        return sub_api("subscribe", *args, **kwargs)

    def check_subscribed(self, *args, **kwargs):
        """
        Check if a user is subscribed to notifications for new project releases.

        Args:
            manager (str): package manager name (e.g. PyPI)
            package (str): package name
        Returns:
            (bool): True if subscribed to given package, else False
        """
        return sub_api("check_subscribed", *args, **kwargs)

    def update_subscribe(self, *args, **kwargs):
        """
        Update the options for a subscription.

        Args:
            manager (str): package manager name (e.g. PyPI)
            package (str): package name
            include_prerelease (bool): default = True. Include prerelease notifications.

        Returns:
            response (dict): dict of project info from libraries.io
            # make so a 404 for not found returns a nice message
            # maybe return a message if 304 is reponse (not updated)
        """
        return sub_api("update_subscribe", *args, **kwargs)

    def unsubscribe(self, *args, **kwargs):
        """
        Stop receiving release notifications from a project.

        Args:
            manager (str): package manager name (e.g. PyPI)
            package (str): package name

        Returns:
            response (str or int): response header status from libraries.io
        """

        return sub_api("delete_subscribe", *args, **kwargs)


# From the command line you can call any public function by name with arguments
if __name__ == "__main__":
    fire.Fire(Subscribe)

    # subs = Subscribe()

    # sub = subs.list_subscribed()
    # print(sub)

    # x = subs.subscribe(manager="pypi", package="pandas")
    # print(x)

    # a = subs.unsubscribe(manager="pypi", package="pandas")
    # print(a)

    # y = subs.check_subscribed("pypi", "numpy")
    # print(y)

    # z = subs.update_subscription(manager="pypi", package="plotly",
    # include_prerelease="False")
    # print(z)



# make_request.pyß
from requests.exceptions import HTTPError
from pybraries.helpers import sess, clear_params


def make_request(url, kind):
    """ call api server 

        Args:
            url (str): base url to call
            kind (str): get, post, put, or delete
        Returns:
            response from libraries.io
    
    """

    try:
        if kind == "get":
            r = sess.get(url)
            r.raise_for_status()
            r_val = r.json()
        if kind == "post":
            r = sess.post(url)
            r.raise_for_status()
            r_val = "successfully subscribed"
        if kind == "put":
            r = sess.put(url)
            r.raise_for_status()
            r_val = "successfully updated"
        if kind == "delete":
            r = sess.delete(url)
            r.raise_for_status()
            r_val = "successfully unsubscribed"
        return r_val

    except HTTPError as http_err:
        print(f"HTTP error occurred: {http_err}")
    except Exception as err:
        print(f"Other error occurred: {err}")



# search_helpers.py
from pybraries.helpers import sess, clear_params
from pybraries.make_request import make_request


def search_api(action, *args, **kwargs):
    """
    build and call for search 

    Args:
        action (str): function action name
        filters (list): list of strings 
        sort (str): to sort by. Options
        *args (str): positional arguments
        **kwargs (str): keyword arguments
    Returns:
        (list): list of dicts response from libraries.io.
            according to page and per page 
        Many are dicts or list of dicts.
    """

    url_end_list = ["https://libraries.io/api"]  # start of list to build url
    more_args = []  # for unpacking args
    url_combined = ""  # final string url
    kind = "get"  # type of request

    if action == "special_project_search":
        url_end_list.append("search?")

        # package seems to be ignored by the libraries.io API
        # - bug in docs or their api
        # if "package" in kwargs:
        #    url_end_list.append(kwargs["package"])

        if kwargs:
            if "filters" in kwargs:
                filts = dict(kwargs["filters"])
                if "manager" in filts:
                    filts["platforms"] = filts.pop("manager")
                sess.params = {**sess.params, **filts}
            if "sort" in kwargs:
                sess.params["sort"] = kwargs["sort"]

        url_combined = "/".join(url_end_list)
        response = make_request(url_combined, kind)

        clear_params()
        return response

    if action == "platforms":
        url_end_list.append("platforms")

    if "pproject" in action:
        if kwargs:
            if "manager" in kwargs:
                url_end_list.append(kwargs["manager"])
            if "package" in kwargs:
                url_end_list.append(kwargs["package"])
        if args:
            more_args = [arg for arg in args]
            url_end_list = url_end_list + more_args

        if action == "pproject_dependencies":
            url_end_list.append(
                "latest"
            )  # could make option to subscribe to other versions
            url_end_list.append("dependencies")

        if action == "pproject_dependents":
            url_end_list.append("dependendents")

        if action == "pproject_dependent_repositories":
            url_end_list.append("dependendent_repositories")

        if action == "pproject_contributors":
            url_end_list.append("contributors")

        if action == "pproject_sourcerank":
            url_end_list.append("sourcerank")

        if action == "pproject_usage":
            url_end_list.append("usage")

    if "repository" in action:
        if kwargs:
            if "host" in kwargs:
                url_end_list.append(kwargs["host"])
            if "owner" in kwargs:
                url_end_list.append(kwargs["owner"])
            if "repo" in kwargs:
                url_end_list.append(kwargs["repo"])
        if args:
            more_args = [arg for arg in args]
            url_end_list = url_end_list + more_args

        if action == "repository_dependencies":
            url_end_list.append("dependencies")

        if action == "repository_projects":
            url_end_list.append("projects")

    if "user" in action:
        if kwargs:
            if "host" in kwargs:
                url_end_list.append(kwargs["host"])
            if "user" in kwargs:
                url_end_list.append(kwargs["user"])
        if args:
            more_args = [arg for arg in args]
            url_end_list = url_end_list + more_args
            print(url_end_list)

        if action == "user_repositories":
            url_end_list.append("repositories")

        if action == "user_packages":
            url_end_list.append("projects")

        if action == "user_packages_contributions":
            url_end_list.append("project-contributions")

        if action == "user_repositories_contributions":
            url_end_list.append("repository-contributions")

        if action == "user_dependencies":
            url_end_list.append("dependencies")

    url_combined = "/".join(url_end_list)
    url_end_list = []
    response = make_request(url_combined, kind)
    return response



# search.py
import fire
from pybraries.search_helpers import search_api
from pybraries.set_pages import set_pages


class Search(object):
    """Class for wrapping the libraries.io API for 
    platform, project, repo, and user GET actions"""

    def __init__(self):
        pass

    def platforms(self, *args, **kwargs):
        """
        Return a list of supported package managers.

        Args:

        Returns:
            list: list of dicts response from libraries.io
        """

        return search_api("platforms", *args, **kwargs)

    def project(self, *args, **kwargs):
        """
        Return information about a package and its versions from a platform (e.g. PyPI).

        Args:
            manager (str): package manager
            package (str): package name
        Returns:
            dict: Information about the project from libraries.io
        """

        return search_api("pproject", *args, **kwargs)

    def project_dependencies(self, *args, **kwargs):
        """
        Get a list of dependencies for a version of a project.

        Returns latest version info.

        Args:
            manager (str): package manager
            package (str): package name
            version (str): package version
        Returns:
            (dict): response from libraries.io
        """

        return search_api("pproject_dependencies", *args, **kwargs)

    def project_dependents(self, *args, **kwargs):
        """
        Get packages that have at least one version that depends on a given project.

        Args:
            manager (str): package manager
            package (str): package name
            version (str): package version
        Returns:
            (list): list of dicts response from libraries.io
        """

        return search_api("pproject_dependendents", *args, **kwargs)

    def project_dependent_repositories(self, *args, **kwargs):
        """
        Get repositories that depend on a given project.

        Args:
            manager (str): package manager
            package (str): package name
        Returns:
            (list): list of dicts response from libraries.io
        """

        return search_api("pproject_dependendent_repositories", *args, **kwargs)

    def project_contributors(self, *args, **kwargs):
        """
        Get users that have contributed to a given project.

        Args:
            manager (str): package manager
            package (str): package name
        Returns:
            (list): list of dicts response from libraries.io
        """

        return search_api("pproject_contributors", *args, **kwargs)

    def project_sourcerank(self, *args, **kwargs):
        """
        Get breakdown of SourceRank score for a given project.

        Args:
            manager (str): package manager
            package (str): package name
        Returns:
            dict: sourcerank info response from libraries.io
        """

        return search_api("pproject_sourcerank", *args, **kwargs)

    def project_usage(self, *args, **kwargs):
        """
        Get breakdown of version usage for a given project.

        Args:
            manager (str): package manager
            package (str): package name
        Returns:
            (dict): verion usage info response from libraries.io
        """

        return search_api("pproject_usage", *args, **kwargs)

    def project_search(self, *args, **kwargs):
        """
        Search for projects.

        ** Accepts keyword arguments only right now. \
            API may change to accept a list of dicts in the future.**

        Args:
            filters (dict): optional dict of form
                dict(languages="python", keywords="data", \
                licenses="my_license", manager="pypi")

            sort (str): (optional) one of rank, stars, 
                dependents_count, dependent_repos_count, 
                latest_release_published_at, contributions_count, created_at
            
        Returns:
            (list): list of dicts of project info from libraries.io
        """

        return search_api("special_project_search", *args, **kwargs)

    def repository(self, *args, **kwargs):
        """
        Return information about a reposiotory and its versions.

        Args:
            host (str): host host name (e.g. GitHub)
            owner (str): owner
            repo (str): repo
        Returns:
            response (list): list of dicts response from libraries.io
        """

        return search_api("repository", *args, **kwargs)

    def repository_dependencies(self, *args, **kwargs):
        """
        Return information about a repository's dependencies.

        Args:
            host (str): host provider name (e.g. GitHub)
            owner (str): owner
            repo (str): repo
        Returns:
            response (dict): dict response from libraries.io
        """

        return search_api("repository_dependencies", *args, **kwargs)

    def repository_projects(self, *args, **kwargs):
        """
        Get a list of packages referencing the given repository.

        Args:
            host (str): host provider name (e.g. GitHub)
            owner (str): owner
            repo (str): repo
        Returns:
            response (list): list of dicts response from libraries.io
        """

        return search_api("repository_projects", *args, **kwargs)

    def user(self, *args, **kwargs):
        """
        Return information about a user.

        Args:
            host (str): host (e.g. github)
            user (str): username
        Returns:
            response (dict): response from libraries.io
        """
        return search_api("user", *args, **kwargs)

    def user_repositories(self, *args, **kwargs):
        """
        Return information about a user's repos.

        Args:
            host (str): host (e.g. github)
            user (str): username
        Returns:
            (list): list of dicts response from libraries.io
        """
        return search_api("user_repositories", *args, **kwargs)

    def user_packages(self, *args, **kwargs):
        """
        Return information about packages using a user's repos.

        Args:
            host (str): host (e.g. github)
            user (str): username
        Returns:
            (list): package with info in list of dicts 
            response from libraries.io
        """
        return search_api("user_packages", *args, **kwargs)

    def user_packages_contributions(self, *args, **kwargs):
        """
        Return information about packages a user has contributed to.

        Args:
            host (str): host (e.g. github)
            user (str): username
        Returns:
            (list): list of dicts response from libraries.io
        """
        return search_api("user_packages_contributions", *args, **kwargs)

    def user_repository_contributions(self, *args, **kwargs):
        """
        Return information about repositories a user has contributed to.

        Args:
            host (str): host (e.g. github)
            user (str): username
        Returns:
            (list): list of dicts response from libraries.io
        """
        return search_api("user_repositories_contributions", *args, **kwargs)

    def user_dependencies(self, *args, **kwargs):
        """
        Return Get a list of unique packages user's repositories' dependencies.

        Ordered by frequency of use in those repositories.

        Args:
            host (str): host (e.g. github)
            user (str): username
        Returns:
            (list): list of dicts response from libraries.io
        """
        return search_api("user_dependencies", *args, **kwargs)


# From the command line you can call any public function by name with arguments
if __name__ == "__main__":
    fire.Fire(Search)

    # manually testing actions
    api = Search()

    t = api.user("github", "discdiver")
    print(t)

    # x = set_pages(1, 3)
    # print(x)

    # d = api.project_search(
    #    sort="latest_release_published_at",
    #    filters=dict(keywords="visualization", manager="pypi"),
    # )
    # print(d)



from pybraries.subscribe import *
from pybraries.search import *
from .helpers import sess
from .subscription_helpers import sub_api
from .search_helpers import search_api
from .set_pages import set_pages
from .make_request import make_request
